;---------------------------------------------------
; TI-Menu.inc
; Re-usable menu code for OS-82.
; (c) 1999 Vasantha Crabb <_merlin@operamail.com>
;
; See file TI-Menu.txt for more information.
;---------------------------------------------------

; These macros address the menu data structure
#DEFINE MENU_LEN        ix+$00
#DEFINE MENU_DEFAULT    ix+$01
#DEFINE MENU_SEL        ix+$02
#DEFINE MENU_TITLE      ix+$03

;---------------------
; SUB for handling a menu
;---------------------

RunMenu:
    ld      de,(PROGRAM_ADDR)   ; Add the program offset
    add     hl,de
    push    hl                  ; Get it into ix
    pop     ix

    push    hl
    ROM_CALL(CLEARTEXT)         ; Clear the text screen
    pop     hl

    ld      b,(hl)              ; Get the number of items in b

    inc     hl                  ; Point hl at the title
    inc     hl
    inc     hl

    ld      de,$0000            ; Move to the top left corner
    ld      (CURSOR_ROW),de
    set     3,(iy+$05)          ; White on black for title
    ROM_CALL(D_ZT_STR)          ; Display the title

    ld      c,1
    res     3,(iy+$05)          ; Black on white for the options
MenuDispLoop:
    ld      a,c                 ; Put item number in a
    CALL_(MenuDispItemNo)       ; Show item number

    ROM_CALL(D_ZT_STR)          ; Display the item's text
    inc     c                   ; Move down a line
    djnz    MenuDispLoop        ; Loop

    ld      a,(MENU_DEFAULT)    ; Start on default item
    CALL_(MenuNewItem)          ; Get the marker onto the screen

MenuHandLoop:
    res     4,(iy+$09)          ; In case the user hit ON
    ROM_CALL(_GETKEY)           ; Get a key and handle it

    cp      K_QUIT              ; The user can exit at any time
    CALL_Z(ClearOut)

    cp      U_ARROW             ; Select previous menu item
    CALL_Z(MenuUpHandler)

    cp      D_ARROW             ; Select next menu item
    CALL_Z(MenuDwnHandler)

    cp      K_ENTER             ; User's made up their mind
    jr      z,MenuRetItem

    cp      K_CLEAR             ; User wants out
    jr      z,MenuRetZero

    cp      144                 ; See if they've hit an item number
    jr      c,MenuHandLoop
    sub     143                 ; Change keycode to item number
    ld      b,a                 ; Save this for now
    ld      a,(MENU_LEN)        ; See if it's on the menu
    cp      b
    jr      c,MenuHandLoop      ; Loop if it isn't
    ld      a,b                 ; Otherwise we return
    ret

MenuRetItem:
    ld      a,(MENU_SEL)        ; Put the selection in a
    ret                         ; Return

;---------------------
; SUB for handling DOWN key on a menu
;---------------------

MenuDwnHandler:
    ld      a,(MENU_SEL)        ; Get the current menu item
    cp      (MENU_LEN)          ; Return if we're on the last item
    jr      z,MenuRetZero

    push    af                  ; Save item number
    CALL_(MenuDispItemNo)       ; Show item number (normal text)

    pop     af                  ; Restore item number
    inc     a                   ; Increment menu item
MenuNewItem:                    ; For code re-use
    ld      (MENU_SEL),a        ; Store the new menu item

    set     3,(iy+$05)          ; White text on black
    CALL_(MenuDispItemNo)       ; Show item number (inverse text)
    res     3,(iy+$05)          ; Black text on white

MenuRetZero:
    xor     a
    ret                         ; Return

;---------------------
; SUB for handling UP key on a menu
;---------------------

MenuUpHandler:
    ld      a,(MENU_SEL)        ; Get the current menu item
    cp      1                   ; Return if we're on the first item
    jr      z,MenuRetZero

    push    af                  ; Save item number
    CALL_(MenuDispItemNo)       ; Show item number (normal text)

    pop     af                  ; Restore item number
    dec     a                   ; Decrement menu item
    jr      MenuNewItem         ; Re-use code

;---------------------
; SUB for showing menu item number
;---------------------

MenuDispItemNo:
    ld      d,$00               ; Cursor column 1
    ld      e,a                 ; Cursor row a
    ld      (CURSOR_ROW),de
    add     a,'0'               ; Convert item number to character
    ROM_CALL(TX_CHARPUT)        ; Put the character on the screen
    ld      a,':'               ; Print a colon
    ROM_CALL(TX_CHARPUT)
    ret                         ; Return

#IFNDEF ClearOut
ClearOut:
    ld      SP,(OldSP)          ; Kill excess stack items
    ret                         ; Return to shell
#ENDIF
